/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * Laborator 3
 * Programarea in C sub AVR, Interfatare LCD alfanumeric, Comunicatie seriala RS232
 *********************************************************************************/

#include <stdio.h>
#include <avr\io.h>
#include <avr\interrupt.h>
#include <string.h>

/************************************************************************
 * DEFINE-uri pentru parametrizarea modulului LCD
 ************************************************************************/
#define LcdDATA_DDR		DDRA			// Portul pe care conectam firele de date la LCD-ul
#define LcdDATA_PORT	PORTA
#define LcdDATA_PIN		PINA

#define LcdCMD_DDR		DDRA			// Portul pe care conectam firele de comenzi la LCD
#define LcdCMD_PORT		PORTA
#define LcdCMD_PIN		PINA

#define LcdD4			PA0				// Pin-ul pentru firul de date D4 de pe LCD
#define LcdD5			PA1				// Pin-ul pentru firul de date D5 de pe LCD
#define LcdD6			PA2				// Pin-ul pentru firul de date D6 de pe LCD
#define LcdD7			PA3				// Pin-ul pentru firul de date D7 de pe LCD
#define LcdRS			PA4				// Pinul pentru selectare operatie (LCD)
#define LcdRW			PA5				// Pinul pentru Read/ Write (LCD)
#define LcdE			PA6				// Pinul de Enable (LCD)
#define Led				PD7

#define LCD_INSTR_4wire 		0x2C 	// 4 fire date, font 5x8
#define LCD_INSTR_display 		0x0C 	// Display On, Cursor On, Blinking On ( 1 Display Cursor Blink )
#define LCD_INSTR_clearDisplay 	0x01 	// Clear Display
#define LCD_INSTR_returnHome 	0x02 	// Return Cursor and LCD to Home Position
#define LCD_INSTR_nextLine 		0xC0 	// Return Cursor and LCD to Home Position
/************************************************************************
 * DEFINE-uri pentru parametrizarea modulului keypad
 ************************************************************************/

#define keyPORT PORTC           //Keypad Port
#define keyDDR DDRC         //Data Direction Register
#define keyPIN PINC         //Keypad Port Pins
#define col1 PC4                //Column1 PortC.4
#define col2 PC6                //Column2 PortC.6
#define col3 PC2                //Column3 PortC.2
#define linie1 PC5
#define linie2 PC0
#define linie3 PC1
#define linie4 PC3

#define TRUE 1
#define FALSE 0

unsigned char keyval; //A variable
int sec = 0;
int min = 0;
int ore = 0;
int sec1 = 0;
int min1 = 0;
int ore1 = 0;
int day = 26;
int month = 5;
int year = 2009;
int day1 = 0;
int month1 = 0;
int year1 = 0;
int armat = 0;
int last_ora = 0;
int last_min = 0;
int last_day = 0;
int last_month = 0;
int last_year = 0;
int last_senz = 0;
int declansat = 0;
unsigned int temp=0;

#define LedVerde PB0
#define LedRosu PB3
#define Light PB1
#define Senzor PB7
#define Claxon PB5

/************************************************************************
 * REDIRECTARE stdin, stdout pentru receptie, transmisie seriala
 ************************************************************************/
int uart_putchar(char c, FILE *unused);
char uart_getchar(FILE *unused);
FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
FILE mystdin = FDEV_SETUP_STREAM(NULL, uart_getchar, _FDEV_SETUP_READ);
unsigned int old = 0;
int state = 0;
int contor = 0;
int light_count = 0;
/************************************************************************
 * API LCD.. implementare este in partea de jos a fisierului
 ************************************************************************/
void LCD_init(); // Initializare modul LCD.. trebuie apelata inainte de a se face orice operatie cu LCD-ul
void LCD_writeInstruction(unsigned char _instruction); // Trimite o instructiune catre lcd (vezi datasheet)
void LCD_writeData(unsigned char _data); // Trimite date catre LCD pentru afisare
void LCD_write(unsigned char _byte); // trimite un bute catre LCD la modul general (nu conteaza daca e instructiune sau date)
void LCD_waitNotBusy(); // Asteptam pana cand lcd-ul devine disponibil pt o noua comanda
void LCD_print(char* _msg); // Afiseaza imformatia pe LCD (doar 1 linie, primele 16 caractere)
void LCD_print2(char* _msg1, char* _msg2); // Afisare pe 2 lini pe LCD
void LCD_printDecimal2u(unsigned int _n); // Afisare numar in baza 10 pe LCD
void LCD_printHexa(unsigned int _n); // Afisare numar in baza 16 pe LCD
void LCD_waitInstructions(unsigned char _instructions); // Asteapta un numar de cicli de tact.. loop

// initializarea convertorului analog digital
void ADC_init() {
	DDRA &= ~_BV(PA7);
	// AVCC cu condesator pe AREF; PA7 ca input single ended; rezultat left-adjusted (8 biti de date)
	ADMUX = (1 << REFS0) | (1 << ADLAR) | (1 << MUX2) | (1 << MUX1) | (1
			<< MUX0);
	// activeaza ADC si intreruperea de conversie. Prescaler de ceas la maximum
	ADCSRA = (1 << ADEN) | (1 << ADIE) | (1 << ADPS2) | (1 << ADPS1) | (1
			<< ADPS0);
}

//initializare keypad
void key_init() {

	keyDDR = _BV(linie1) | _BV(linie2) | _BV(linie3) | _BV(linie4);//liniile sunt de iesire
	//	keyDDR=0xFF;
	//	keyPORT=0;
	//keyPORT=(~_BV(linie1))&(~_BV(linie2))&(~_BV(linie3))&(~_BV(linie4));
	keyPORT = 0xFF;
	//	PORTC= _BV(col1) | _BV(col2)| _BV(col3);//coloanele sunt initializate pe 1
	// keyport = 0x0F;
}

void initTimer1() // Initializare timer 1 compare match A pentru a avea un timer la nivel de 1 secunda, 16MHz
{
	OCR1A = 15624;
	TCCR1A = 0;
	TCCR1B = (1 << WGM12) | (1 << CS12) | (1 << CS10);
	TIMSK |= _BV(OCIE1A);
}
void initTimer2() // Initializare timer 2 compare match A pentru a avea un timer la nivel de 1 secunda, 16MHz
{
	OCR2 = 156;
	TCCR2 = (1 << WGM21) | (1 << CS22) | (1 << CS20) | (1 << CS21);;
	TIMSK |= _BV(OCIE2);
}

unsigned char get_key() {

	//Make rows low one by one
	keyPORT = ~(_BV(linie1));

	if (!(keyPIN&_BV(col1))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col1)))
			;

		return '1';
	}
	if (!(keyPIN&_BV(col2))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col2)))
			;

		return '2';
	}
	if (!(keyPIN&_BV(col3))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col3)))
			;

		return '3';
	}
	keyPORT = ~(_BV(linie2));

	if (!(keyPIN&_BV(col1))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col1)))
			;

		return '4';
	}
	if (!(keyPIN&_BV(col2))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col2)))
			;

		return '5';
	}
	if (!(keyPIN&_BV(col3))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col3)))
			;

		return '6';
	}
	keyPORT = ~(_BV(linie3));

	if (!(keyPIN&_BV(col1))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col1)))
			;

		return '7';
	}
	if (!(keyPIN&_BV(col2))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col2)))
			;

		return '8';
	}
	if (!(keyPIN&_BV(col3))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col3)))
			;

		return '9';
	}
	keyPORT = ~(_BV(linie4));

	if (!(keyPIN&_BV(col1))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col1)))
			;

		return '*';
	}
	if (!(keyPIN&_BV(col2))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col2)))
			;

		return '0';
	}
	if (!(keyPIN&_BV(col3))) {
		//check COL1

		//wait for release
		while (!(keyPIN&_BV(col3)))
			;

		return '#';
	}

	return 0; //return false if no key pressed
}

// intreruperea de conversie
SIGNAL(SIG_ADC)
{
	unsigned int data;

	data = ADCH;
	temp=(5 * data * 100)/256;
	//	data1=(5*data*100)/256;
	//a=get_key();
	printf("  %u C\n",temp);
	printf("   %02d-%02d-%04d",day,month,year);

}
char buffer1[32];
char buffer2[32];
int len1 = 0;
int len2 = 0;
int stateP = 0;
int stare = 0;
int state21 = 0;
int cont = 0;
int state31 = 0;
int state41 = 0;
int state991 = 0;
int state701 = 0;
int state51 = 0;

SIGNAL(SIG_OUTPUT_COMPARE2)//se apeleaza de 100 ori pe secunda
{
	char a;
	TCCR2 = 0;
	TIMSK &= ~(_BV(OCIE2));

	cont++;
	if(cont==20) {
		cont=0;

		if(armat) {
			PORTB &= ~(_BV(LedVerde));
			PORTB |= (_BV(LedRosu));
			if((PINB&_BV(Senzor)) || temp>=50) {
				PORTD|=_BV(Led);
				PORTB|=_BV(Claxon);
				if(declansat==0) {
					last_min=min;
					last_ora=ore;
					last_day=day;
					last_month=month;
					last_year=year;
					declansat=1;
					if(temp>=50){
						last_senz=2;
					} else {
						last_senz=1;
					}
					
				}
			}
		} else {
			PORTB &= ~(_BV(LedRosu));
			PORTB |= (_BV(LedVerde));
			PORTD&=~(_BV(Led));
			PORTB&=~(_BV(Claxon));
		}

		a=get_key();
		if(a!=0) {
			PORTB |= _BV(Light);
			light_count=0;
		}
		if(light_count==10) {
			PORTB &= ~(_BV(Light));
			light_count=0;
		}
		if(armat) {
			if(stare==70) {
				if(state701==0 && a!='#') {
					//state701=1;
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					printf("%02d:%02d:%02d",ore,min,sec);
					//	printf("a");
					ADCSRA |= (1<<ADSC);
					memset(buffer2,0,32);
					len2=0;
				}

				if(a=='#' && state701==0) {
					state701=2;
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					printf("Parola:\n");
					initTimer2();
					return;
				}
				else if(state701==2) {
					if(a!=0)
					{
						//LCD_writeInstruction(LCD_INSTR_clearDisplay);
						if(a!='#')
						{

							printf("%c",a);
							buffer2[len2]=a;
							len2++;
							buffer2[len2]=0;
							initTimer2();
							return;
						}

						else
						{
							if(!strncmp(buffer1,buffer2,len1)) {

								state991=0;
								state701=0;
								stare=1;
								armat=0;
							}
							else {
								state701=0;

							}
							memset(buffer2,0,32);
							len2=0;
							initTimer2();
							return;

						}
					}
				}
			}

		} else {

			declansat=0;
			if(stare==1) {
				LCD_writeInstruction(LCD_INSTR_clearDisplay);
				printf("%02d:%02d:%02d",ore,min,sec);
				ADCSRA |= (1<<ADSC);
				if(a=='#') {
					stare=2;
					state21=0;
					stateP=0;
					initTimer2();
					return;
				}
				else if(a=='*') {
					stare=99;
					//printf("*");
					initTimer2();
					return;
				}
			}
			else if(stare==99) {//armeaza
				if(state991==0) {
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					printf("Armeaza ???\nN              Y");
					state991=1;
				}
				else if(state991==1) {
					if(a=='#') {
						LCD_writeInstruction(LCD_INSTR_clearDisplay);
						printf("Parola:\n");
						state991=2;
						initTimer2();
						return;
					}
					else if(a=='*') {
						state991=0;
						stare=1;
						initTimer2();
						return;
					}
				}
				else if(state991==2) {

					if(a!=0)
					{

						if(a!='#')
						{

							printf("%c",a);
							buffer2[len2]=a;
							len2++;
							buffer2[len2]=0;

						}

						else
						{
							if(!strncmp(buffer1,buffer2,len1)) {

								state991=0;//parola setata
								state701=0;
								stare=70;
								armat=1;
								LCD_writeInstruction(LCD_INSTR_clearDisplay);
							}
							else {
								state991=0;
								stare=1;

							}
							memset(buffer2,0,32);
							len2=0;

						}
					}

				}

			}
			else if (stare==2) {

				if(a=='#' && state21==0) {
					state21=1;

					//stare=5;
					initTimer2();
					return;
				}
				else if(a=='*'&& state21==0) {
					stare=1;
					printf("*");
					initTimer2();
					return;
				}
				else if(a=='6' && state21==0) {
					stare=3;

					state31=0;
					initTimer2();
					return;
				}
				if(state21==0 && stare==2) {

					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					printf("Schimba parola\n|              >");
				}
				if(state21==1) {//set password
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					printf("Intr pass:");
					if(a!=0)
					{

						state21=2;
						stateP=1;
						LCD_writeInstruction(LCD_INSTR_clearDisplay);
					}
				}
				if(state21==2) {
					if(stateP==1) {
						if(a!=0)
						{
							if(a!='#')
							{
								if(a=='*')
								{
									;

								}
								else
								{
									printf("%c",a);
									buffer1[len1]=a;
									len1++;
									buffer1[len1]=0;
								}

							}

							else
							{

								stateP=2;//
								LCD_writeInstruction(LCD_INSTR_clearDisplay);
								printf("Repeta parola:");

							}
						}
					}
					else if(stateP==2) {

						if(a!=0)
						{
							LCD_writeInstruction(LCD_INSTR_clearDisplay);
							if(a!='#')
							{
								if(a=='*')
								{
									;

								}
								else
								{
									printf("%c",a);
									buffer2[len2]=a;
									len2++;
									buffer2[len2]=0;
								}

							}

							else
							{

								if(!strncmp(buffer1,buffer2,len1)) {

									stateP=0;//parola setata
									state21=0;
								}
								else {
									memset(buffer1,0,32);
									len1=0;
									stateP=0;
									state21=1;
								}
								memset(buffer2,0,32);
								len2=0;

							}
						}
					}

				}
			}
			else if( stare==3) {//setare ora

				if(state31==0) {
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					printf("Seteaza ora\n<              >");
				}
				if(a=='#' && state31==0) {

					state31=1;
					//stare=5;
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					printf("Introd ora: \n");
					//printf("
					initTimer2();
					return;
				}
				else if(a=='*'&& state31==0) {
					stare=1;
					initTimer2();
					return;
				}
				else if(a=='4' && state31==0) {
					stare=2;
					state21=0;
					initTimer2();
					return;
				}
				else if(a=='6' && state31==0) {
					stare=4;
					state41=0;
					initTimer2();
					return;
				}
				else if(a!=0 && a!='#' && a!='*') {
					if(state31==1) {
						//	LCD_writeInstruction(LCD_INSTR_clearDisplay);
						printf("%c",a);
						ore1=10*(a-'0');
						state31=2;
					}
					else if(state31==2) {
						ore1+=a-'0';
						//	LCD_writeInstruction(LCD_INSTR_clearDisplay);
						printf("%c:",a);
						state31=3;
					}
					else if(state31==3) {
						min1=10*(a-'0');
						//LCD_writeInstruction(LCD_INSTR_clearDisplay);
						printf("%c",a);
						state31=4;
					}
					else if(state31==4) {
						min1+=a-'0';
						//	LCD_writeInstruction(LCD_INSTR_clearDisplay);
						printf("%c",a);
						state31=5;
					}
				}
				else if(state31==5 && a=='#') {
					ore=ore1;
					//	LCD_writeInstruction(LCD_INSTR_clearDisplay);
					//printf("5");
					min=min1;
					state31=0;
					stare=3;

				}

			}
			else if( stare==4) {//setare data

				if(state41==0) {
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					printf("Seteaza data\n<              >");
				}
				if(a=='#' && state41==0) {

					state41=1;
					//stare=5;
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					printf("Introd data: \n");
					//printf("
					initTimer2();
					return;
				}
				else if(a=='*'&& state41==0) {
					stare=1;
					initTimer2();
					return;
				}
				else if(a=='4' && state41==0) {
					stare=3;
					state31=0;
					initTimer2();
					return;
				}
				else if(a=='6' && state41==0) {
					stare=5;
					state51=0;
					initTimer2();
					return;
				}
				else if(a!=0 && a!='#' && a!='*') {
					if(state41==1) {
						printf("%c",a);
						day1=10*(a-'0');
						state41=2;
					}
					else if(state41==2) {
						day1+=a-'0';
						printf("%c-",a);
						state41=3;
					}
					else if(state41==3) {
						month1=10*(a-'0');
						printf("%c",a);
						state41=4;
					}
					else if(state41==4) {
						month1+=a-'0';
						printf("%c-",a);
						state41=5;
					}
					else if(state41==5) {
						year1=1000*(a-'0');
						printf("%c",a);
						state41=6;
					}
					else if(state41==6) {
						year1+=100*(a-'0');
						printf("%c",a);
						state41=7;
					}
					else if(state41==7) {
						year1+=10*(a-'0');
						printf("%c",a);
						state41=8;
					}
					else if(state41==8) {
						year1+=a-'0';
						printf("%c",a);
						state41=9;
					}
				}
				else if(state41==9 && a=='#') {
					day=day1;
					month=month1;
					year=year1;
					state41=0;
					stare=4;

				}

			}
			else if( stare==5) {//afisare ultima declansare

				if(state51==0) {
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					printf("Ultima declansare\n<              |");
				}
				if(a=='#' && state51==0) {

					state51=1;

				}
				else if(a=='*'&& state51==0) {
					stare=1;
					initTimer2();
					return;
				}
				else if(a=='4' && state51==0) {
					stare=4;
					state41=0;
					initTimer2();
					return;
				}

				else if(state51==1) {
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					if(last_senz==1) {
						printf("usa: ");
					} else {
						printf("temp: ");
					}
					printf("   %02d:%02d\n",last_ora,last_min);
					printf("   %02d-%02d-%04d",last_day,last_month,last_year);
					if(a=='*') {
						state51=0;
						stare=5;
					}

				}

			}
		}
	}
	initTimer2();
}

// Exemplu de functie ce se apeleaza la interval de 1 secunda, intrerupere generata de timerul 1 (vezi laboratorul 2)
SIGNAL(SIG_OUTPUT_COMPARE1A)
{
	// Cod ce se apeleaza 1 data pe secunda
	// Exemplu de implemenare in C pentru laboratorul nr 2 (ASM)

	//contor++;
	//	if(contor==10)
	//	{

	sec++;
	light_count++;
	if(sec==60)
	{
		sec=0;
		min++;
		if(min==60)
		{
			min=0;
			ore++;
			if(ore==24)
			{
				ore=0;
				day++;
				if((month==1 || month==3||month==5||month==7||month==8||month==10||month==12) && day==32){
					day=1;
					month++;
				}
				else if((month==4||month==6||month==9||month==11 ) &&day==31){
					day=1;
					month++;
				}
				else if(month==2 && day==29){
					day=1;
					month++;
				}
				if(month==13){
					month=1;
					year++;
				}
			}
		}
	}

}

/************************************************************************
 * PROGRAMUL PRINCIPAL
 ************************************************************************/
int main(void) {

	// Initializare


	stdout = &mystdout;
	stdin = &mystdin;

	DDRD = _BV(Led);
	PORTD &= ~_BV(Led);
	key_init();
	// Activare sistem de intreruperi
	//	DDRB=0xFF;
	//	PORTB=0;
	DDRB |= _BV(Light) | _BV(LedRosu) | _BV(LedVerde)|_BV(Claxon);
	DDRB &= ~(_BV(Senzor));
	PORTB |= _BV(Senzor);
	PORTB &= (~_BV(LedRosu)) & (~_BV(LedRosu))&(~_BV(Claxon));
	//PORTB |= _BV(Light);
	//	DDRC=0xFF;
	//PORTC=0x0F;
	//	while(1){
	//		;
	//	}
	sei();
	// Functie pentru initializarea LCD-ului si afisarea mesajului WELCOME
	LCD_init();

	//	DDRC=0xFF;
	//	PORTC=0xFF;
	//	PORTC |=_BV(PC2);
	//	while(1){
	//	;
	//	}
	//	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	//	LCD_print("WELCOME");

	ADC_init();

	memcpy(buffer1, "1234", 4);
	len1 = 4;
	// Asteptam dupa un mesaj si apoi il afisam pe LCD, dupa care reluam ciclul
	//	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	//	printf("DINAMO !!!");
	initTimer2();
	initTimer1();

	stare = 1;
	for (;;) {
		// cer ADC-ului sa faca o conversie
		//	ADCSRA |= (1<<ADSC);
		//for (i = 0; i < 10000; i++);

		//scanf("%c",&a);


	}
	return 0;
}

int uart_putchar(char c, FILE *unused) {
	if (c == '\n') {
		LCD_writeInstruction(LCD_INSTR_nextLine);
		return 0;
	}
	LCD_writeData(c);
	return 0;
}

char uart_getchar(FILE *unused) {
	char a = get_key();
	if (a == '#') {
		return '\n';
	}

	return a;
}

/************************************************************************
 * IMPLEMENTARE API LCD.. implementare este in partea de jos a fisierului
 ************************************************************************/

void LCD_init() {
	LcdDATA_DDR |= (1 << LcdD4) | (1 << LcdD5) | (1 << LcdD6) | (1 << LcdD7); // setam pinii de date ca pini de iesire
	LcdCMD_DDR |= (1 << LcdRS) | (1 << LcdRW) | (1 << LcdE); // setam pinii de comenzi ca pini de iesire

	LCD_waitNotBusy();

	LcdCMD_PORT &= ~(1 << LcdRS); // Setam linia RS pe low
	LcdCMD_PORT &= ~(1 << LcdRW); // Setam linia RW pe low (acum suntem in modul de trimis instructiuni)
	LcdDATA_PORT &= ~(1 << LcdD4) & ~(1 << LcdD6) & ~(1 << LcdD7); // Specificam ca vrem 4 fire de date, prima comanda (LcdD5 activ, restul nu)
	LcdDATA_PORT |= (1 << LcdD5); // setam pinii de comenzi ca pini de iesire

	LcdCMD_PORT |= (1 << LcdE); // Setam linia E(nable) pe high; aceasta ii specifica LCD-ului sa preia datele
	LCD_waitInstructions(6); // Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1 << LcdE); // Setam linia E(nable) pe low; transferul s-a terminat

	LCD_writeInstruction(LCD_INSTR_4wire); // Incarcam comanda: 4 bit data, 2 lines, 5x8 font
	LCD_writeInstruction(LCD_INSTR_display); // Display On, Cursor On, Blinking On
	LCD_writeInstruction(0x06); // Increment, no shift
	LCD_writeInstruction(0x01); // Clear Display
}

void LCD_writeInstruction(unsigned char _instruction) {
	LCD_waitNotBusy();
	LcdCMD_PORT &= ~(1 << LcdRS) & ~(1 << LcdRW);
	LCD_write(_instruction);
	//TODO
	// asteptam ca LCD-ul sa fie liber sa primeasca comenzile
	// setam pinul RS pe low.. low=instructiuni, high=date
	// setam pinul RW pe low (suntem in modul de comenzi acum)
	// apelam procedura ce trimite byte-ul pe firele de date
}

void LCD_writeData(unsigned char _data) {
	LCD_waitNotBusy();
	LcdCMD_PORT |= 1 << LcdRS;
	LcdCMD_PORT &= ~(1 << LcdRW);
	LCD_write(_data);
	//TODO
	// asteptam ca LCD-ul sa fie liber sa primeasca comenzile
	// setam pinul RS pe high
	// setam pinu RW pe low (suntem in modul de date acum)
	// apelam procedura ce trimite byte-ul pe firele de date
}

void LCD_write(unsigned char _byte) {
	unsigned char _byte2;

	_byte2 = _byte >> 4;

	LcdDATA_PORT &= ~(1 << LcdD4);
	if (bit_is_set(_byte2, 0))
		LcdDATA_PORT |= (1 << LcdD4);

	LcdDATA_PORT &= ~(1 << LcdD5);
	if (bit_is_set(_byte2, 1))
		LcdDATA_PORT |= (1 << LcdD5);

	LcdDATA_PORT &= ~(1 << LcdD6);
	if (bit_is_set(_byte2, 2))
		LcdDATA_PORT |= (1 << LcdD6);

	LcdDATA_PORT &= ~(1 << LcdD7);
	if (bit_is_set(_byte2, 3))
		LcdDATA_PORT |= (1 << LcdD7);

	LcdCMD_PORT |= (1 << LcdE); // Setam Pinul E pe high
	LCD_waitInstructions(6); // Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1 << LcdE); // Setam Pinul E pe low

	LCD_waitInstructions(6); // Asteptam o perioada de timp T

	LcdDATA_PORT &= ~(1 << LcdD4);
	if (bit_is_set(_byte, 0))
		LcdDATA_PORT |= (1 << LcdD4);

	LcdDATA_PORT &= ~(1 << LcdD5);
	if (bit_is_set(_byte, 1))
		LcdDATA_PORT |= (1 << LcdD5);

	LcdDATA_PORT &= ~(1 << LcdD6);
	if (bit_is_set(_byte, 2))
		LcdDATA_PORT |= (1 << LcdD6);

	LcdDATA_PORT &= ~(1 << LcdD7);
	if (bit_is_set(_byte, 3))
		LcdDATA_PORT |= (1 << LcdD7);

	LcdCMD_PORT |= (1 << LcdE); // Setam Pinul E pe high
	LCD_waitInstructions(6); // Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1 << LcdE); // Setam Pinul E pe low
}

void LCD_waitNotBusy() {
	unsigned char _loop = 1;

	while (_loop) {
		LcdDATA_DDR &= ~(1 << LcdD4 | 1 << LcdD5 | 1 << LcdD6 | 1 << LcdD7); // Setam pinii de date de la LCD pe in pt a citi busy flag
		LcdDATA_PORT &= ~(1 << LcdD4 | 1 << LcdD5 | 1 << LcdD6 | 1 << LcdD7); // Dezactivam pullup resistor pentru pinii de in

		LcdCMD_PORT &= ~(1 << LcdE); // Setam pin-ul e pe low; ar trebui sa fie deja low, doar ne asiguram
		LcdCMD_PORT &= ~(1 << LcdRS); // Setam pinul RS pe low
		LcdCMD_PORT |= (1 << LcdRW); // Setam pinul RW pe high (acum suntem in modul de interogare busy/adr)

		LcdCMD_PORT |= (1 << LcdE); // Setam Pinul E pe high
		LCD_waitInstructions(6); // Asteptam o perioada de timp T
		_loop = LcdDATA_PIN & (1 << LcdD7); // Citim busy flag-ul
		LcdCMD_PORT &= ~(1 << LcdE); // Setam Pinul E pe low

		LCD_waitInstructions(6); // Asteptam o perioada de timp T

		LcdCMD_PORT |= (1 << LcdE); // Setam Pinul E pe high
		LCD_waitInstructions(6); // Asteptam o perioada de timp T
		LcdCMD_PORT &= ~(1 << LcdE); // Setam Pinul E pe low

		LcdDATA_DDR |= (1 << LcdD4 | 1 << LcdD5 | 1 << LcdD6 | 1 << LcdD7); // Setam Portul de LCD ca port de iesire la loc
	}
}

void LCD_printDecimal2u(unsigned int _n) {
	unsigned char tmp = 0;

	// Extragem sutele
	while (_n >= 100)
		_n -= 100;

	while (_n >= 10) {
		tmp++;
		_n -= 10;
	}

	LCD_writeData(tmp + '0');
	LCD_writeData(_n + '0');
}

void LCD_printHexa(unsigned int _n) {
	unsigned char _tmp = _n >> 4;
	if (_tmp > 9)
		_tmp += 'A' - 10;
	else
		_tmp += '0';
	LCD_writeData(_tmp);
	_tmp = _n & 0x0F;
	if (_tmp > 9)
		_tmp += 'A' - 10;
	else
		_tmp += '0';
	LCD_writeData(_tmp);
}

void LCD_print(char* _msg) {
	unsigned char i = 0;
	for (; _msg[i] != 0 && i < 16; i++)
		LCD_writeData(_msg[i]);
}

void LCD_print2(char* _msg1, char* _msg2) {
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print(_msg1);
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_print(_msg2);
}

void LCD_waitInstructions(unsigned char _instructions) {
	while (_instructions--)
		;
}

